searchData={"items":[{"type":"behaviour","title":"Construct","doc":"Construct internally divided into three components:\n\n  * `Construct` — defining structures;\n  * `Construct.Cast` — making structure instances;\n  * `Construct.Type` — type-coercion and custom type behaviour.","ref":"Construct.html"},{"type":"behaviour","title":"Construct definition - Construct","doc":"defmodule StructureName do\n      use Construct, struct_opts\n\n      structure do\n        include AnotherStructure\n        field name, type, options\n      end\n    end\n\n`struct_opts` is options passed to `c:make/2` and `c:make!/2`, described in `Construct.Cast.make/3`.\n\nWhen you type `use Construct` — library bootstrapped few functions with `Construct` behaviour:\n\n  * `c:make/2` — just an alias to `Construct.Cast.make/3`;\n  * `c:make!/2` — alias to `c:make/2` but throws `Construct.MakeError` exception if provided params are invalid;\n  * `c:cast/2` — alias to `c:make/2` too, for follow `Construct.Type` behaviour and use defined structure as type.","ref":"Construct.html#module-construct-definition"},{"type":"callback","title":"Construct.cast/2","doc":"Alias to `c:make/2`, used to follow `c:Construct.Type.cast/1` callback.\n\nTo use this structure as custom type.","ref":"Construct.html#c:cast/2"},{"type":"function","title":"Construct.construct?/1","doc":"Checks if provided module is Construct module","ref":"Construct.html#construct?/1"},{"type":"macro","title":"Construct.field/3","doc":"Defines field on the structure with given name, type and options.\n\nChecks definition validity at compile time by name, type and options.\nFor custom types checks for module existence and `c:Construct.Type.cast/1` callback.\n\nIf field definition is invalid for some reason — it throws an `Construct.DefinitionError`\nexception with detailed reason.","ref":"Construct.html#field/3"},{"type":"macro","title":"Options - Construct.field/3","doc":"* `:default` — sets default value for that field:\n\n    * The default value is calculated at compilation time, so don't use expressions like\n      DateTime.utc_now or Ecto.UUID.generate as they would then be the same for all structures;\n\n    * Value from params is compared with default value before and after type cast;\n\n    * If you pass `field :a, type, default: nil` and `make(%{a: nil})` — type coercion will\n      not be used, `nil` compares with default value and just appends that value to structure;\n\n    * If field doesn't exist in params, it will use default value.\n\n    By default this option is unset. Notice that you can't use functions as a default value.","ref":"Construct.html#field/3-options"},{"type":"macro","title":"Construct.include/2","doc":"Includes provided structure and checks definition for validity at compile-time.","ref":"Construct.html#include/2"},{"type":"macro","title":"Options - Construct.include/2","doc":"* `:only` - (integer) specify fields that should be taken from included module,\n    throws an error when field doesn't exist in provided module.\n\nIf included structure is invalid for some reason — this macro throws an\n`Construct.DefinitionError` exception with detailed reason.","ref":"Construct.html#include/2-options"},{"type":"callback","title":"Construct.make/2","doc":"Alias to `Construct.Cast.make/3`.","ref":"Construct.html#c:make/2"},{"type":"callback","title":"Construct.make!/2","doc":"Alias to `c:make/2`, but raises an `Construct.MakeError` exception if params have errors.","ref":"Construct.html#c:make!/2"},{"type":"macro","title":"Construct.structure/1","doc":"Defines a structure.","ref":"Construct.html#structure/1"},{"type":"macro","title":"Construct.structure_compile_hook/2","doc":"No doc at this time, should be written for 3.0.0 release","ref":"Construct.html#structure_compile_hook/2"},{"type":"function","title":"Construct.types_of!/1","doc":"Collect types from defined Construct module to map","ref":"Construct.html#types_of!/1"},{"type":"type","title":"Construct.t/0","doc":"","ref":"Construct.html#t:t/0"},{"type":"module","title":"Construct.Cast","doc":"Module to make structure instance from provided params.\n\nYou can use it standalone, without defining structure, by providing types and params to `make/3`.","ref":"Construct.Cast.html"},{"type":"function","title":"Construct.Cast.make/3","doc":"Function to compose structure instance from params:\n\n    defmodule User do\n      use Construct\n\n      structure do\n        field :name\n      end\n    end\n\n    iex> make(User, %{name: \"john doe\"})\n    {:ok, %User{name: \"john doe\"}}\n\n    iex> make(User, name: \"john doe\")\n    {:ok, %User{name: \"john doe\"}}\n\nAlso you can use it as standalone complex type-coercion by providing types and params:\n\n    iex> make(%{name: {:string, []}}, %{\"name\" => \"john doe\"})\n    {:ok, %{name: \"john doe\"}}\n\n    iex> make(%{name: :string}, %{\"name\" => \"john doe\"})\n    {:ok, %{name: \"john doe\"}}\n\n    iex> make([name: :string], %{\"name\" => \"john doe\"})\n    {:ok, %{name: \"john doe\"}}\n\n    iex> make(%{age: {:integer, [default: 18]}}, %{\"age\" => \"42\"})\n    {:ok, %{age: 42}}\n\n    iex> make(%{age: {:integer, [default: 18]}}, %{})\n    {:ok, %{age: 18}}\n\n    iex> types = %{title: {:string, []}, comments: {{:array, :string}, default: []}}\n    iex> make(types, %{title: \"article\", comments: [\"awesome\", \"great!\", \"whoa!\"]})\n    {:ok, %{title: \"article\", comments: [\"awesome\", \"great!\", \"whoa!\"]}}\n\n    iex> make(%{user: %{name: :string, age: {:integer, default: 21}}}, %{\"user\" => %{\"name\" => \"john\"}})\n    {:ok, %{user: %{name: \"john\", age: 21}}}","ref":"Construct.Cast.html#make/3"},{"type":"type","title":"Construct.Cast.options/0","doc":"","ref":"Construct.Cast.html#t:options/0"},{"type":"type","title":"Construct.Cast.type/0","doc":"","ref":"Construct.Cast.html#t:type/0"},{"type":"type","title":"Construct.Cast.types/0","doc":"","ref":"Construct.Cast.html#t:types/0"},{"type":"module","title":"Construct.Hooks.Fasten","doc":"","ref":"Construct.Hooks.Fasten.html"},{"type":"behaviour","title":"Construct.Type","doc":"Type-coercion module, originally copied and modified from\n[Ecto.Type](https://github.com/elixir-ecto/ecto/blob/master/lib/ecto/type.ex)\nand behaviour to implement your own types.","ref":"Construct.Type.html"},{"type":"behaviour","title":"Defining custom types - Construct.Type","doc":"defmodule CustomType do\n      @behaviour Construct.Type\n\n      def cast(value) do\n        {:ok, value}\n      end\n    end","ref":"Construct.Type.html#module-defining-custom-types"},{"type":"function","title":"Construct.Type.builtin/0","doc":"","ref":"Construct.Type.html#builtin/0"},{"type":"function","title":"Construct.Type.builtinc/0","doc":"","ref":"Construct.Type.html#builtinc/0"},{"type":"callback","title":"Construct.Type.cast/1","doc":"Casts the given input to the custom type.","ref":"Construct.Type.html#c:cast/1"},{"type":"function","title":"Construct.Type.cast/2","doc":"Behaves like `cast/3`, but without options provided to nested types.","ref":"Construct.Type.html#cast/2"},{"type":"function","title":"Construct.Type.cast/3","doc":"Casts a value to the given type.\n\n    iex> cast(:any, \"whatever\")\n    {:ok, \"whatever\"}\n\n    iex> cast(:any, nil)\n    {:ok, nil}\n    iex> cast(:string, nil)\n    :error\n\n    iex> cast(:integer, 1)\n    {:ok, 1}\n    iex> cast(:integer, \"1\")\n    {:ok, 1}\n    iex> cast(:integer, \"1.0\")\n    :error\n\n    iex> cast(:float, 1.0)\n    {:ok, 1.0}\n    iex> cast(:float, 1)\n    {:ok, 1.0}\n    iex> cast(:float, \"1\")\n    {:ok, 1.0}\n    iex> cast(:float, \"1.0\")\n    {:ok, 1.0}\n    iex> cast(:float, \"1-foo\")\n    :error\n\n    iex> cast(:boolean, true)\n    {:ok, true}\n    iex> cast(:boolean, false)\n    {:ok, false}\n    iex> cast(:boolean, \"1\")\n    {:ok, true}\n    iex> cast(:boolean, \"0\")\n    {:ok, false}\n    iex> cast(:boolean, \"whatever\")\n    :error\n\n    iex> cast(:string, \"beef\")\n    {:ok, \"beef\"}\n    iex> cast(:binary, \"beef\")\n    {:ok, \"beef\"}\n\n    iex> cast(:decimal, Decimal.from_float(1.0))\n    {:ok, Decimal.from_float(1.0)}\n    iex> cast(:decimal, Decimal.new(\"1.0\"))\n    {:ok, Decimal.from_float(1.0)}\n    iex> cast(:decimal, 1.0)\n    {:ok, Decimal.from_float(1.0)}\n    iex> cast(:decimal, \"1.0\")\n    {:ok, Decimal.from_float(1.0)}\n\n    iex> cast({:array, :integer}, [1, 2, 3])\n    {:ok, [1, 2, 3]}\n    iex> cast({:array, :integer}, [\"1\", \"2\", \"3\"])\n    {:ok, [1, 2, 3]}\n    iex> cast({:array, :string}, [1, 2, 3])\n    :error\n    iex> cast(:string, [1, 2, 3])\n    :error\n\n    iex> cast({Construct.Types.Enum, [:a, :b, :c]}, :a)\n    {:ok, :a}\n    iex> cast({Construct.Types.Enum, [:a, :b, :c]}, :d)\n    {:error, passed_value: :d, valid_values: [:a, :b, :c]}","ref":"Construct.Type.html#cast/3"},{"type":"function","title":"Construct.Type.primitive?/1","doc":"Checks if we have a primitive type.\n\n    iex> primitive?(:string)\n    true\n    iex> primitive?(Another)\n    false\n\n    iex> primitive?({:array, :string})\n    true\n    iex> primitive?({:array, Another})\n    true\n\n    iex> primitive?([Another, {:array, :integer}])\n    false","ref":"Construct.Type.html#primitive?/1"},{"type":"type","title":"Construct.Type.builtin/0","doc":"","ref":"Construct.Type.html#t:builtin/0"},{"type":"type","title":"Construct.Type.cast_ret/0","doc":"","ref":"Construct.Type.html#t:cast_ret/0"},{"type":"type","title":"Construct.Type.custom/0","doc":"","ref":"Construct.Type.html#t:custom/0"},{"type":"type","title":"Construct.Type.t/0","doc":"","ref":"Construct.Type.html#t:t/0"},{"type":"behaviour","title":"Construct.TypeC","doc":"Provides a way to create parametrized types.\n\nSee `Construct.Types.Enum` implementation for more information.","ref":"Construct.TypeC.html"},{"type":"callback","title":"Construct.TypeC.castc/2","doc":"","ref":"Construct.TypeC.html#c:castc/2"},{"type":"module","title":"Construct.Types.CommaList","doc":"Extracts list of separated by comma values from string.\n\nYou can use it alone, just for splitting values:\n\n    defmodule Structure do\n      use Construct do\n        field :values, Construct.Types.CommaList\n      end\n    end\n\n    iex> Structure.make!(values: \"foo,bar,baz,42\")\n    %Structure{values: [\"foo\", \"bar\", \"baz\", \"42\"]}\n\n    iex> Structure.make!(values: [\"foo\", 42])\n    %Structure{values: [\"foo\", 42]}\n\nAlso you can compose it with other types:\n\n    defmodule UserInfoRequest do\n      use Construct do\n        field :user_ids, [Construct.Types.CommaList, {:array, :integer}]\n      end\n    end\n\n    iex> UserInfoRequest.make!(%{user_ids: \"1,2,42\"})\n    %UserInfoRequest{user_ids: [1, 2, 42]}\n\n    iex> UserInfoRequest.make(%{user_ids: \"1,foo\"})\n    {:error, %{user_ids: :invalid}}","ref":"Construct.Types.CommaList.html"},{"type":"module","title":"Construct.Types.Enum","doc":"Implements an abstract enum type.","ref":"Construct.Types.Enum.html"},{"type":"module","title":"Usage - Construct.Types.Enum","doc":"defmodule MyApp.Order do\n      use Construct do\n        field :type, {Construct.Types.Enum, [:delivery, :pickup]}\n      end\n    end\n\nThen you can validate that `:type` field accepts only specified\nvalues.\n\n    iex> MyApp.Order.make(%{type: :delivery})\n    {:ok, %MyApp.Order{type: :delivery}}\n    iex> MyApp.Order.make(%{type: :other})\n    {:error, %{type: passed_value: :other, valid_values: [:delivery, :pickup]}}","ref":"Construct.Types.Enum.html#module-usage"},{"type":"module","title":"Construct.Types.UUID","doc":"Checks that provided binary is UUID-like string:\n\n    defmodule Structure do\n      use Construct do\n        field :value, Construct.Types.UUID\n      end\n    end\n\n    iex> Structure.make!(value: \"fd4ddf80-a7d9-4af8-b46c-26fc4566d92c\")\n    %Structure{value: \"fd4ddf80-a7d9-4af8-b46c-26fc4566d92c\"}\n\n    iex> Structure.make(value: \"invalid\")\n    {:error, %{value: :invalid}}","ref":"Construct.Types.UUID.html"},{"type":"exception","title":"Construct.DefinitionError","doc":"","ref":"Construct.DefinitionError.html"},{"type":"exception","title":"Construct.Error","doc":"","ref":"Construct.Error.html"},{"type":"exception","title":"Construct.MakeError","doc":"","ref":"Construct.MakeError.html"},{"type":"extras","title":"Construct","doc":"# Construct [![Hex.pm](https://img.shields.io/hexpm/v/construct.svg)](https://hex.pm/packages/construct)\n\n---\n\nLibrary for dealing with data structures\n\n---\n\n* [Installation](#installation)\n* [Usage](#usage)\n* [Types](#types)\n* [Construct definition](#construct-definition)\n* [Errors while making structures](#errors-while-making-structures)\n\n---","ref":"readme.html"},{"type":"extras","title":"Installation - Construct","doc":"1. Add `construct` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:construct, \"~> 2.0\"}]\nend\n```\n\n2. Ensure `construct` is started before your application:\n\n```elixir\ndef application do\n  [applications: [:construct]]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Construct","doc":"Suppose you have some user input from several sources (DB, HTTP request, WebSocket), and you will need to process that data into something type-validated, like User entity. With this library you can define a type-validated structure for this entity:\n\n```elixir\ndefmodule User do\n  use Construct do\n    field :name\n    field :age, :integer\n  end\nend\n```\n\nAnd use it to cast your data into something identical, to prevent type coercion in different places of your code. Like this:\n\n```elixir\niex> User.make(%{\"name\" => \"John Doe\", \"age\" => \"37\"})\n{:ok, %User{age: 37, name: \"John Doe\"}}\n```\n\nPretty neat, yeah? But what if you need more complex type? We have a solution!\n\n```elixir\ndefmodule Answer do\n  @behaviour Construct.Type\n\n  def cast(\"yes\"), do: {:ok, true}\n  def cast(\"no\"), do: {:ok, false}\n  def cast(_), do: {:error, :invalid_answer}\nend\n```\n\nAnd use it in your structure like this:\n\n```elixir\ndefmodule Quiz do\n  use Construct do\n    field :user_id, :integer\n    field :answers, {:array, Answer}\n  end\nend\n```\n\n```elixir\niex> Quiz.make(%{user_id: 42, answers: [\"yes\", \"no\", \"no\", \"yes\"]})\n{:ok, %Quiz{answers: [true, false, false, true], user_id: 42}}\n```\n\n> What if we need to parse 'optimized' query string from URL, like list of user ids separated by a comma? Do we need to create a custom type for each boxed type?\n\nNo! Just use type composition feature:\n\n```elixir\ndefmodule CommaList do\n  @behaviour Construct.Type\n\n  def cast(\"\"), do: {:ok, []}\n  def cast(v) when is_binary(v), do: {:ok, String.split(v, \",\")}\n  def cast(v) when is_list(v), do: {:ok, v}\n  def cast(_), do: :error\nend\n\ndefmodule SearchFilterRequest do\n  use Construct do\n    field :user_ids, [CommaList, {:array, :integer}], default: []\n  end\nend\n```\n\n(Use `CommaList` type from [construct_types](https://github.com/ExpressApp/construct_types/blob/master/lib/types/comma_list.ex) package).\n\n```elixir\niex> SearchFilterRequest.make(%{\"user_ids\" => \"1,2,42\"})\n{:ok, %SearchFilterRequest{user_ids: [1, 2, 42]}}\n```\n\nAlso we have `default` option in our `user_ids` field:\n\n```elixir\niex> SearchFilterRequest.make(%{})\n{:ok, %SearchFilterRequest{user_ids: []}}\n```\n\n> What if I have a lot of identical code?\n\nYou can use already defined structures as types:\n\n```elixir\ndefmodule Comment do\n  use Construct do\n    field :text\n  end\nend\n\ndefmodule Post do\n  use Construct do\n    field :title\n    field :comments, {:array, Comment}\n  end\nend\n\niex> Post.make(%{title: \"Some article\", comments: [%{\"text\" => \"cool!\"}, %{text: \"awesome!!!\"}]})\n{:ok, %Post{comments: [%Comment{text: \"cool!\"}, %Comment{text: \"awesome!!!\"}], title: \"Some article\"}}\n```\n\nAnd include repeated fields in structures:\n\n```elixir\ndefmodule PK do\n  use Construct do\n    field :primary_key, :integer\n  end\nend\n\ndefmodule Timestamps do\n  use Construct do\n    field :created_at, :utc_datetime, default: &DateTime.utc_now/0\n    field :updated_at, :utc_datetime, default: nil\n  end\nend\n\ndefmodule User do\n  use Construct do\n    include PK\n    include Timestamps\n\n    field :name\n  end\nend\n\niex> User.make(%{name: \"John Doe\", primary_key: 42})\n{:ok,\n %User{created_at: #DateTime<2018-10-14 20:43:06.595119Z>, name: \"John Doe\",\n  primary_key: 42, updated_at: nil}}\n\niex> User.make(%{name: \"John Doe\", created_at: \"2015-01-23 23:50:07\", primary_key: 42})\n{:ok,\n %User{created_at: #DateTime<2015-01-23 23:50:07Z>, name: \"John Doe\",\n  primary_key: 42, updated_at: nil}}\n```\n\n> What if I don't want to define module to make a nested field?\n\n`field` macro can `do` it for you:\n\n```elixir\ndefmodule User do\n  use Construct do\n    field :name do\n      field :first\n      field :last, :string, default: nil\n    end\n  end\nend\n\niex> User.make(name: %{first: \"John\"})\n{:ok, %User{name: %User.Name{first: \"John\", last: nil}}}\n```\n\nConstruct tries to fit in Elixir as much as it possible:\n\n```elixir\ndefmodule ComplexDefaults do\n  use Construct do\n    field :required\n\n    field :nested do\n      field :key, :string, default: \"nesting 1\"\n\n      field :nested do\n        field :key, :string, default: \"nesting 2\"\n      end\n    end\n  end\nend\n\niex> %ComplexDefaults{}\n** (ArgumentError) the following keys must also be given when building struct ComplexDefaults: [:required]\n    expanding struct: ComplexDefaults.__struct__/1\n\niex> %ComplexDefaults{required: 1}\n%ComplexDefaults{\n  nested: %ComplexDefaults.Nested{\n    key: \"nesting 1\",\n    nested: %ComplexDefaults.Nested.Nested{key: \"nesting 2\"}\n  },\n  required: 1\n}\n```\n\n> What if I want to use union types?\n\nUse custom types:\n\n```elixir\ndefmodule User do\n  use Construct do\n    field :id, :integer\n    field :name\n    field :age, :integer\n  end\nend\n\ndefmodule Bot do\n  use Construct do\n    field :id, :integer\n    field :name\n    field :version\n  end\nend\n\ndefmodule Author do\n  @behaviour Construct.Type\n\n  # here's the trick, just choose the type by yourself, based on keys or value in specific field.\n  # but be careful, because there can be atoms and strings in keys!\n  def cast(%{\"age\" => _} = v), do: User.make(v)\n  def cast(%{\"version\" => _} = v), do: Bot.make(v)\n  def cast(_), do: :error\nend\n\ndefmodule Post do\n  use Construct do\n    field :author, Author\n  end\nend\n\niex> Post.make(%{\"author\" => %{}})\n{:error, %{author: :invalid}}\n\niex> Post.make(%{\"author\" => %{\"age\" => \"420\"}})\n{:error, %{author: %{id: :missing, name: :missing}}}\n\niex> Post.make(%{\"author\" => %{\"id\" => \"42\", \"name\" => \"john doe\", \"age\" => \"420\"}})\n{:ok, %Post{author: %User{age: 420, id: 42, name: \"john doe\"}}}\n\niex> Post.make(%{\"author\" => %{\"id\" => \"42\", \"name\" => \"john doe\", \"version\" => \"1.0.0\"}})\n{:ok, %Post{author: %Bot{id: 42, name: \"john doe\", version: \"1.0.0\"}}}\n```\n\n> How can I serialize my structures with Jason?\n\nUse `@derive` attribute and `derive` option for nested fields:\n\n```elixir\ndefmodule Server do\n  @derive {Jason.Encoder, only: [:name, :operating_system]}\n\n  use Construct do\n    field :name\n    field :password\n\n    field :operating_system, derive: Jason.Encoder do\n      field :name, :string\n      field :arch, :string, default: \"x86\"\n    end\n  end\nend\n\niex> {:ok, server} = Server.make(name: \"example\", password: \"secret\", operating_system: %{name: \"MacOS\"})\n{:ok,\n %Server{\n   name: \"example\",\n   operating_system: %Server.OperatingSystem{arch: \"x86\", name: \"MacOS\"},\n   password: \"secret\"\n }}\n\niex> Jason.encode!(server)\n\"{\\\"name\\\":\\\"example\\\",\\\"operating_system\\\":{\\\"arch\\\":\\\"x86\\\",\\\"name\\\":\\\"MacOS\\\"}}\"\n```","ref":"readme.html#usage"},{"type":"extras","title":"Types - Construct","doc":"","ref":"readme.html#types"},{"type":"extras","title":"Primitive types - Construct","doc":"* `t()`:\n  * integer\n  * float\n  * boolean\n  * string\n  * binary\n  * decimal\n  * utc_datetime\n  * naive_datetime\n  * date\n  * time\n  * any\n  * array\n  * map\n  * struct\n* `{:array, t()}`\n* `{:map, t()}`\n* `[t()]`","ref":"readme.html#primitive-types"},{"type":"extras","title":"Complex (custom) types - Construct","doc":"You can use Ecto custom types like Ecto.UUID or implement by yourself:\n\n```elixir\ndefmodule CustomType do\n  @behaviour Construct.Type\n\n  @spec cast(term) :: {:ok, term} | {:error, term} | :error\n  def cast(value) do\n    {:ok, value}\n  end\nend\n```\n\nNotice that `cast/1` can return error with reason, this behaviour is supported only by Struct and you can't use types defined using Construct in Ecto schemas.","ref":"readme.html#complex-custom-types"},{"type":"extras","title":"Construct definition - Construct","doc":"```elixir\ndefmodule User do\n  use Construct, struct_opts\n\n  structure do\n    include module_name\n\n    field name\n    field name, type\n    field name, type, field_opts\n  end\nend\n```\n\nWhere:\n\n* `use Construct, struct_opts` where:\n  * `struct_opts` — options passed to every `make/2` and `make!/2` calls as default options;\n* `include module_name` where:\n  * `module_name` — is struct module, that validates for existence in compile time;\n* `field name, type, field_opts` where:\n  * `name` — atom;\n  * `type` — primitive or custom type, that validates for existence in compile time;\n  * `field_opts`.","ref":"readme.html#construct-definition"},{"type":"extras","title":"Errors while making structures - Construct","doc":"When you provide invalid data to your structures you can get tuple with errors as maps:\n\n```elixir\niex> Post.make\n{:error, %{comments: :missing, title: :missing}}\n\niex> Post.make(%{comments: %{}, title: :test})\n{:error, %{comments: :invalid, title: :invalid}}\n\niex> Post.make(%{comments: [%{}], title: \"what the title?\"})\n{:error, %{comments: %{text: :missing}}}\n```\n\nOr receive an exception with invalid data:\n\n```elixir\niex> Post.make!\n** (Construct.MakeError) %{comments: {:missing, nil}, title: {:missing, nil}}\n    iex:10: Post.make!/2\n\niex> Post.make!(%{comments: %{}, title: :test})\n** (Construct.MakeError) %{comments: {:invalid, %{}}, title: {:invalid, :test}}\n    iex:10: Post.make!/2\n\niex> Post.make!(%{comments: [%{}], title: \"what the title?\"})\n** (Construct.MakeError) %{comments: %{text: {:missing, [nil]}}}\n    iex:10: Post.make!/2\n```\n\n---","ref":"readme.html#errors-while-making-structures"},{"type":"extras","title":"Contributing - Construct","doc":"1. Fork it\n2. Create your feature branch (`git checkout -b my-new-feature`)\n3. Commit your changes (`git commit -am 'add some feature'`)\n4. Push to the branch (`git push origin my-new-feature`)\n5. Create new Pull Request","ref":"readme.html#contributing"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}